#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/**** sample: interval timer :begin ****/
#include <time.h>
#include <sys/time.h>
/**** sample: interval timer :end ****/

#define MAIN_C
#include "exitfail.h"

#define SERIAL_BAUDRATE B115200
#define BUF_SIZE 2048

static int serial_fd = -1;
static struct termios old_tio;

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

/**** sample: RN4020 command for IoT Smart: begin ****/
static unsigned char cmd_ism_init[] = "CHW,0018,2F0303\n"; /* set iot smart: init */
static unsigned char cmd_ism_select_sensor[] = "CHW,0018,01037F\n"; /* set iot smart: select sensor */
static unsigned char cmd_ism_measure_mode[] = "CHW,0018,040304\n"; /* set iot smart: measure mode */
static unsigned char cmd_ism_fast_interval[] = "CHW,0018,06046400\n"; /* set iot smart: fast interval */
static unsigned char cmd_ism_slow_interval[] = "CHW,0018,05040A00\n"; /* set iot smart: slow interval */
//static unsigned char cmd_ism_save[] = "CHW,0018,2F0301\n"; /* set iot smart: save setting */
static unsigned char cmd_ism_start[] = "CHW,0018,200301\n"; /* set iot smart: start */
static unsigned char cmd_ism_stop[] = "CHW,0018,200300\n"; /* set iot smart: stop */
static unsigned char cmd_disconnect[] = "K\n"; /* disconnect */
/**** sample: RN4020 command for IoT Smart: end ****/

/**** sample: RN4020/IoT Smart response: begin ****/
/*static unsigned char res_connect[] = "Connected"; *//* rn4020 response: connect */
static unsigned char res_disconnect[] = "Connection End"; /* rn4020 response: disconnect */
static unsigned char res_notify[] = "Notify,0012"; /* iot smart response: notify from custom1*/
static unsigned char res_notify_status[] = "Notify,0012,E0"; /* iot smart response: notify from custom1 and staus */
/**** sample: RN4020/IoT Smart response: end ****/

/**** sample: IoT Smart motion data: begin ****/
#define ISM_MOTION_SIZE 120

static struct {
	float mag_x;
	float mag_y;
	float mag_z;
	float acc_x;
	float acc_y;
	float acc_z;
	int time_ms;
	int time_s;
	int time_m;
	int time_h;
} ism_motion_data[ISM_MOTION_SIZE];
/**** sample: IoT Smart motion data: end ****/

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

static void restore_serial(void)
{
	int ret;

	ret = tcsetattr(serial_fd, TCSANOW, &old_tio);
	if(ret < 0) exitfail_errno("tcsetattr");
}

static void setup_serial(int fd)
{
	struct termios tio;
	int ret;

	ret = tcgetattr(fd, &old_tio);
	if(ret) exitfail_errno("tcsgetattr");

	if(atexit(restore_serial)) exitfail_errno("atexit");

	memset(&tio, 0, sizeof(tio));
	tio.c_iflag = IGNBRK | IGNPAR;
	tio.c_cflag = CS8 | CLOCAL | CREAD;
	tio.c_cc[VTIME] = 0;
	tio.c_cc[VMIN] = 1;
	ret = cfsetspeed(&tio, SERIAL_BAUDRATE);
	if(ret < 0) exitfail_errno("cfsetspeed");

	ret = tcflush(fd, TCIFLUSH);
	if(ret < 0) exitfail_errno("tcflush");

	ret = tcsetattr(fd, TCSANOW, &tio);
	if(ret) exitfail_errno("tcsetattr");
}

/**** sample: char(HEX) to int: begin ****/ 
int ch2i(unsigned char c) {
	int ret;
	
	ret = 0;

	if(c>='0' && c<='9') ret = (unsigned int)c - (unsigned int)'0';	
	else if(c>='a' && c <='f') ret = (unsigned int)c - (unsigned int)'a' + 10;	
	else if(c>='A' && c <='F') ret = (unsigned int)c - (unsigned int)'A' + 10;
	else ret = -1;

	return ret;	
}
/**** sample: char(HEX) to int: end ****/ 

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};

/**** sample: variable: begin ****/
	ssize_t ret;
       unsigned char buf[BUF_SIZE];
       unsigned char buf1[BUF_SIZE];
	int i, j;
	FILE *fp;
	char env_file[100];
	char motion_file[100];
	int flag_notify, flag_period, flag_1shot, flag_disconnect;
	int cnt_cmp;
	ssize_t analen;
/**** sample: variable: end ****/

/**** sample: variable for senseor data: begin ****/
	float mag_x, mag_y, mag_z;
	float acc_x, acc_y, acc_z;
	int time_ms, time_s, time_m, time_h;
	float pres, hum, temp, uv, amb;
	int date_d, date_m, date_y;
	int cnt_motion;

	int cal_tmp;
/**** sample: for senseor data: end ****/

/**** sample: interval timer: begin ****/
	struct itimerval itimer;
/**** sample: interval timer: end ****/

	exitfail_init();

	if(argc < 3) {
		printf("Usage: %s <device> <id> [0, 1]\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	serial_fd = open(argv[1], O_RDWR | O_NOCTTY);
	if(serial_fd < 0) exitfail_errno("open");

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	setup_serial(serial_fd);

/**** sample: log file name: begin ****/
	for(i = 0; i < 100; i++) {
		env_file[i] = '\0';
		motion_file[i] = '\0';
	}
	sprintf(env_file, "/root/log_ism_env_%s.txt", argv[2]);
	sprintf(motion_file, "/root/log_ism_motion_%s.txt", argv[2]);
/**** sample: log file name: begin ****/

/**** sample: interval timer: begin ****/
	itimer.it_value.tv_sec = itimer.it_interval.tv_sec = 30;
	itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
/**** sample: interval timer: end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Init): begin ****/
	ret = write(serial_fd, cmd_ism_init, sizeof(cmd_ism_init) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_init);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_period != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Init): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Select Sensor): begin ****/
	ret = write(serial_fd, cmd_ism_select_sensor, sizeof(cmd_ism_select_sensor) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_select_sensor);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_period != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Select Sensor): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Mesure Mode): begin ****/
	ret = write(serial_fd, cmd_ism_measure_mode, sizeof(cmd_ism_measure_mode) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_measure_mode);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_period !=  1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
	sleep(10); /* wait for IoT Smart changes mesure mode*/
/**** sample: RN4020 command(CHW to set IoT Smart: Mesure Mode): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Fast Interval): begin ****/
	ret = write(serial_fd, cmd_ism_fast_interval, sizeof(cmd_ism_fast_interval) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_fast_interval);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	cnt_motion = 0;

	while(!terminated && flag_period != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
	sleep(10); /* wait for IoT Smart changes mesure mode*/
/**** sample: RN4020 command(CHW to set IoT Smart: Fast Interval): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Slow Interval): begin ****/
	ret = write(serial_fd, cmd_ism_slow_interval, sizeof(cmd_ism_slow_interval) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_slow_interval);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_period != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
	sleep(10); /* wait for IoT Smart changes mesure mode*/
/**** sample: RN4020 command(CHW to set IoT Smart: Slow Interval): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Start): begin ****/
	ret = write(serial_fd, cmd_ism_start, sizeof(cmd_ism_start) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_start);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	cnt_motion = 0;

	flag_1shot = 0;

	while(!terminated && flag_1shot != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
//			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
				if(flag_period == 1) {
					/**** sample: analize sensor data: begin ****/
					if(buf1[12] == 'F' && buf1[13] == '2') {
/**** <--- Indent Shift: begin ****/

/**** sample: analize sensor data(motion): begin ****/
cal_tmp = (((ch2i(buf1[18]) * 16 + ch2i(buf1[19])) * 16 + ch2i(buf1[16])) * 16 + ch2i(buf1[17]));
if((ch2i(buf1[18]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_x = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[22]) * 16 + ch2i(buf1[23])) * 16 + ch2i(buf1[20])) * 16 + ch2i(buf1[21]));
if((ch2i(buf1[22]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_y = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[26]) * 16 + ch2i(buf1[27])) * 16 + ch2i(buf1[24])) * 16 + ch2i(buf1[25]));
if((ch2i(buf1[26]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_z = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[30]) * 16 + ch2i(buf1[31])) * 16 + ch2i(buf1[28])) * 16 + ch2i(buf1[29]));
if((ch2i(buf1[30]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_x = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

cal_tmp = (((ch2i(buf1[34]) * 16 + ch2i(buf1[35])) * 16 + ch2i(buf1[32])) * 16 + ch2i(buf1[33]));
if((ch2i(buf1[34]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_y = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

cal_tmp = (((ch2i(buf1[38]) * 16 + ch2i(buf1[39])) * 16 + ch2i(buf1[36])) * 16 + ch2i(buf1[37]));
if((ch2i(buf1[38]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_z = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

time_ms = (float)(((ch2i(buf1[42]) * 16 + ch2i(buf1[43])) * 16 + ch2i(buf1[40])) * 16 + ch2i(buf1[41]));
time_s = ch2i(buf1[44]) * 16 + ch2i(buf1[45]);
time_m = ch2i(buf1[46]) * 16 + ch2i(buf1[47]);
time_h = ch2i(buf1[48]) * 16 + ch2i(buf1[49]);
/**** sample: analize sensor data(motion): end ****/

/**** output sensor data(motion) to stdout: begin ****/
/*
printf("\n----------------------------------------------------\n");
printf("%s\n", buf1);
if(buf1[18] == '8' && buf1[19] == '0' && buf1[16] == '0' && buf1[17] == '0')
	printf("Magnetic[uT]: not selected\n");
else 
	printf("Magnetic[uT]: (x, y, z) = (%.2f, %.2f, %.2f)\n", mag_x, mag_y, mag_z);

if(buf1[30] == '8' && buf1[31] == '0' && buf1[28] == '0' && buf1[29] == '0')
	printf("Acceleration[G]: not selected\n");
else
	printf("Acceleration[G]: (x, y, z) = (%.2f, %.2f, %.2f)\n", acc_x, acc_y, acc_z);

printf("Time Stamp: %d:%d:%d (%d[ms])\n", time_h, time_m, time_s, time_ms);
*/
/**** output sensor data(motion) to stdout: end ****/

/**** sample: sensor data(motion) to buffer: begin ****/
ism_motion_data[cnt_motion].mag_x = mag_x;
ism_motion_data[cnt_motion].mag_y = mag_y;
ism_motion_data[cnt_motion].mag_z = mag_z;
ism_motion_data[cnt_motion].acc_x = acc_x;
ism_motion_data[cnt_motion].acc_y = acc_y;
ism_motion_data[cnt_motion].acc_z = acc_z;
ism_motion_data[cnt_motion].time_ms = time_ms;
ism_motion_data[cnt_motion].time_s = time_s;
ism_motion_data[cnt_motion].time_m = time_m;
ism_motion_data[cnt_motion].time_h = time_h;
if(cnt_motion < 100) cnt_motion++;
/**** sample: sensor data(motion) to buffer: end ****/
					}
					if(buf1[12] == 'F' && buf1[13] == '3') {
/**** <--- Indent Shift: begin ****/

/**** sample: analize sensor data(env): begin ****/
pres = (float)(((ch2i(buf1[18]) * 16 + ch2i(buf1[19])) * 16 + ch2i(buf1[16])) * 16 + ch2i(buf1[17])) * 860 / 65535 + 250;
hum = (float)((((ch2i(buf1[22]) * 16 + ch2i(buf1[23])) * 16 + ch2i(buf1[20])) * 16 + ch2i(buf1[21])) - 896) / 64;
temp = (float)((((ch2i(buf1[26]) * 16 + ch2i(buf1[27])) * 16 + ch2i(buf1[24])) * 16 + ch2i(buf1[25])) - 2096) / 50;
uv = (float)(((ch2i(buf1[30]) * 16 + ch2i(buf1[31])) * 16 + ch2i(buf1[28])) * 16 + ch2i(buf1[29])) / (100 * 0.388);
amb = (float)(((ch2i(buf1[34]) * 16 + ch2i(buf1[35])) * 16 + ch2i(buf1[32])) * 16 + ch2i(buf1[33]));
if(uv < 0.814)
  amb = amb / (0.05 * 0.928 * (0.3102 * uv + 0.8525));
else
  amb = amb / (0.05 * 0.928 * (0.03683 * uv + 1.075));
date_d = ch2i(buf1[44]) * 16 + ch2i(buf1[45]); 
date_m = ch2i(buf1[46]) * 16 + ch2i(buf1[47]); 
date_y = ch2i(buf1[48]) * 16 + ch2i(buf1[49]); 
/**** sample: analize sensor data(env): end ****/

/**** output sensor data(env) to stdout: begin ****/
printf("\n****************************************************\n");
printf("\n---------[%s] sensor data(env) --------\n", argv[2]);
printf("%s\n", buf1);
if(buf1[18] == '0' && buf1[19] == '0' && buf1[16] == '0' && buf1[17] == '0')
	printf("Pressure[hPa]: not selected\n");
else 
	printf("Pressure[hPa]: %.2f\n", pres);

if(buf1[22] == '8' && buf1[23] == '0' && buf1[20] == '0' && buf1[21] == '0')
	printf("Humidity[%%RH]: not selected\n");
else 
	printf("Humidity[%%RH]: %.2f\n", hum);

if(buf1[26] == '8' && buf1[27] == '0' && buf1[24] == '0' && buf1[25] == '0')
	printf("Temparature[degC]: not selected\n");
else 
	printf("Temparature[degC]: %.2f\n", temp);

if(buf1[30] == '8' && buf1[31] == '0' && buf1[28] == '0' && buf1[29] == '0')
	printf("UV[mW/cm^2]: not selected\n");
else 
	printf("UV[mW/cm^2]: %.2f\n", uv);

if(buf1[34] == '8' && buf1[35] == '0' && buf1[32] == '0' && buf1[33] == '0')
	printf("Ambient[Lx]: not selected\n");
else
	printf("Ambient[Lx]: %.2f\n", amb);

printf("Date: %d.%d.%d\n", date_y, date_m, date_d);
/**** output sensor data(env) to stdout: end ****/

/**** sample: sensor data(env) to log file for post to heroku: begin ****/
fp = fopen(env_file, "w");

if(buf1[18] == '0' && buf1[19] == '0' && buf1[16] == '0' && buf1[17] == '0')
	fprintf(fp, "pres=None&");
else 
	fprintf(fp, "pres=%.2f&", pres);

if(buf1[22] == '8' && buf1[23] == '0' && buf1[20] == '0' && buf1[21] == '0')
	fprintf(fp, "hum=None&");
else 
	fprintf(fp, "hum=%.2f&", hum);

if(buf1[26] == '8' && buf1[27] == '0' && buf1[24] == '0' && buf1[25] == '0')
	fprintf(fp, "tmp=None&");
else 
	fprintf(fp, "tmp=%.2f&", temp);

if(buf1[30] == '8' && buf1[31] == '0' && buf1[28] == '0' && buf1[29] == '0')
	fprintf(fp, "uv=None&");
else 
	fprintf(fp, "uvi=%.2f&", uv);

if(buf1[34] == '8' && buf1[35] == '0' && buf1[32] == '0' && buf1[33] == '0')
	fprintf(fp, "amb=None&");
else
	fprintf(fp, "lux=%.2f&", amb);
fprintf(fp, "date=%d.%d.%d\n", date_y, date_m, date_d);

fclose(fp);
/**** sample: sensor data(env) to log file for post to heroku: end ****/

/**** sample: output latest sensor data(motion) to stdout: end ****/
printf("\n---------[%s] sensor data(motion) --------\n", argv[2]);
printf("Magnetic[uT]: (x, y, z) = (%.2f, %.2f, %.2f)\n", 
	ism_motion_data[cnt_motion - 1].mag_x, 
	ism_motion_data[cnt_motion - 1].mag_y, 
	ism_motion_data[cnt_motion - 1].mag_z);
printf("Acceleration[G]: (x, y, z) = (%.2f, %.2f, %.2f)\n", 
	ism_motion_data[cnt_motion - 1].acc_x, 
	ism_motion_data[cnt_motion - 1].acc_y, 
	ism_motion_data[cnt_motion - 1].acc_z);
printf("Time Stamp: %d:%d:%d (%d[ms])\n", 
	ism_motion_data[cnt_motion - 1].time_h, 
	ism_motion_data[cnt_motion - 1].time_m, 
	ism_motion_data[cnt_motion - 1].time_s, 
	ism_motion_data[cnt_motion - 1].time_ms);
/**** sample: output latest sensor data(motion) to stdout: end ****/

/**** sample: sensor data(motion) to log file: begin ****/
fp = fopen(motion_file, "w");
for(j = 0; j < cnt_motion; j++)
	fprintf(fp, "time=%d:%d:%d(%d[ms])&mag=(%.2f, %.2f, %.2f)&acc=(%.2f, %.2f, %.2f)\n", 
		ism_motion_data[j].time_h, 
		ism_motion_data[j].time_m, 
		ism_motion_data[j].time_s, 
		ism_motion_data[j].time_ms, 
		ism_motion_data[j].mag_x, 
		ism_motion_data[j].mag_y, 
		ism_motion_data[j].mag_z, 
		ism_motion_data[j].acc_x, 
		ism_motion_data[j].acc_y, 
		ism_motion_data[j].acc_z);
fclose(fp);
cnt_motion = 0;
/**** sample: sensor data(motion) to log file: end ****/
if(*argv[3] != '0') flag_1shot = 1;
/**** <--- Indent Shift: end ****/
					}
					/**** sample: analize sensor data: end ****/
					flag_notify = 0;
					flag_period = 0;
					cnt_cmp = 0;
					analen = 0;
				}
			}
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Start): end ****/

/**** sample: RN4020 command(CHW to set IoT Smart: Stop): begin ****/
	ret = write(serial_fd, cmd_ism_stop, sizeof(cmd_ism_stop) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_ism_stop);

	flag_notify = 0;
	flag_period = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_period != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_period != 1; i++) {
			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."NotNotify" */
			if(flag_notify != 1 && buf[i] == res_notify_status[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_notify == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_notify_status) - 1) flag_notify = 1;
			if(flag_notify == 1 && buf[i] == '.') {
				flag_period = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_period == 1) {
			printf("\n-------- Receive: %s\n", res_notify_status);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(CHW to set IoT Smart: Stop): end ****/

	sleep(1);

/**** sample: RN4020 command(K to disconnect IoT Smart): begin ****/
	ret = write(serial_fd, cmd_disconnect, sizeof(cmd_disconnect) - 1);
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Command: %s", cmd_disconnect);

	flag_disconnect = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated && flag_disconnect != 1) {
		if(setitimer(ITIMER_REAL, &itimer, NULL) < 0) {
			exitfail_errno("setitimer");
			return 0;
		}
	
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_disconnect != 1; i++) {
//			printf("%c", buf[i]);
			/* Not considered about search resoponse. ex."CoConection End" */
			if(buf[i] == res_disconnect[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				analen = 0;
			}
			if(cnt_cmp == sizeof(res_disconnect) - 1) {
				flag_disconnect = 1;
				buf1[analen] = '\0';
			}
		}
		if(flag_disconnect == 1) {
			printf("\n-------- Receive: %s\n", res_disconnect);
//			printf("%s\n", buf1);
		}
	}
/**** sample: RN4020 command(K to disconnect IoT Smart): end ****/
	printf("\n######## Disconnected: IoT Smart ########\n\n");

	return EXIT_SUCCESS;
}

